@@ -1,10 +1,10 @@
 import datetime
 
-__version__ = '0.1.10'
+__version__ = '0.1.14'
 
-__release_day__ = 4
-__release_month_num__ = 8
-__release_year__ = 2021
+__release_day__ = 7
+__release_month_num__ = 4
+__release_year__ = 2022
 
 
 __release_date_object__ = datetime.date(__release_year__, __release_month_num__, __release_day__)
@@ -17,7 +17,7 @@ __author_email__ = 'c.calafiore@essex.ac.uk'
 __maintainer_email__ = 'c.calafiore@essex.ac.uk'
 __all__ = [
     'array', 'check', 'clock', 'combinations', 'directory', 'download',
-    'format', 'image', 'maths', 'mixamo', 'ml', 'pkl', 'plot', 'preprocessing', 'shutdown', 'stats',
+    'format', 'image', 'lists', 'maths', 'mixamo', 'ml', 'pkl', 'plot', 'preprocessing', 'shutdown', 'stats',
     'stimulation', 'strings', 'txt']
 
 
