@@ -35,7 +35,7 @@ def samples_in_arr1_are_in_arr2(arr1, arr2, axis=0):
     arr1 = np.array(arr1)
     type_arr1 = arr1.dtype
     if type_arr1 == object:
-        arr2 = np.array(list(arr1))
+        arr1 = np.array(list(arr1))
 
     arr2 = np.array(arr2)
     type_arr2 = arr2.dtype
@@ -105,7 +105,7 @@ def samples_in_arr1_are_in_arr2(arr1, arr2, axis=0):
     #             indexes_arr1[i_dimension] = np.arange(shape_arr1[i_dimension])
 
     for i_sample in range(n_samples_arr1):
-        indexes_arr1[axis] = i_sample
+        indexes_arr1[axis] = slice(i_sample, i_sample + 1, 1)
         indexes_arr_logical[axis] = i_sample
         arr_logical[tuple(indexes_arr_logical)] = arr1[tuple(indexes_arr1)] == arr2
 
