@@ -1,8 +1,8 @@
 import torch
 import torchvision
-from .device import *
 
-__all__ = ['datasets', 'models', 'set_device', 'features', 'test', 'train']
+
+__all__ = ['torch', 'torchvision', 'datasets', 'models', 'devices', 'preprocess', 'test', 'train']
 
 
 def initiate(names_submodules=None):
