@@ -6,11 +6,14 @@
 # https://pytorch.org/tutorials/beginner/transfer_learning_tutorial.html
 
 
+import torchvision
+import typing
 import math
 import numpy as np
 from ... import array as cp_array
+from ... import directory as cp_directory
 
-__all__ = ['Shifts']
+__all__ = ['Shifts', 'define_transforms', 'define_file_format']
 
 
 class Shifts:
@@ -62,6 +65,7 @@ class Shifts:
             self.unique[v] = self.ranges[v].to_list()
             self.n_unique[v] = self.unique[v].__len__()
 
+        self.n_shifts = n_shifts
         self.__set_n_shifts__(n_shifts)
         self.i = -1
 
@@ -107,7 +111,7 @@ class Shifts:
     def __next__(self):
         self.i += 1
         if self.i < self.n_shifts:
-            self.values_i = self.values[self.i, slice(0, self.n_levels, 1)]
+            self.values_i = self.__getitem__(self.i)
             return self.values_i
         else:
             raise StopIteration
@@ -115,3 +119,66 @@ class Shifts:
     def refresh(self):
         self.__iter__()
         return self.values
+
+    def __getitem__(self, i):
+        self.values_i = self.values[self.i, slice(0, self.n_levels, 1)]
+        return self.values_i
+
+    def __len__(self):
+        if self.n_shifts is None:
+            raise ValueError('self.n_shifts')
+        else:
+            return self.n_shifts
+
+
+def define_transforms(transforms: typing.Union[torchvision.transforms.Compose, None] = None):
+
+    if transforms is None:
+        return torchvision.transforms.Compose([torchvision.transforms.ToTensor()])
+    else:
+        to_tensor_is_not_in_transforms = True
+        for transform_t in transforms.transforms:
+            if type(torchvision.transforms.ToTensor()) == type(transform_t):
+                to_tensor_is_not_in_transforms = False
+                break
+        if to_tensor_is_not_in_transforms:
+            transforms.transforms.append(torchvision.transforms.ToTensor())
+            to_tensor_is_not_in_transforms = True
+
+        return transforms
+
+
+def define_file_format(format_file: str = None, directory_file: str = None):
+
+    # format_file_out = cp_directory.get_extension(directory_file, point=False).lower()
+
+    if format_file is None:
+
+        if directory_file is None:
+            raise ValueError('format_file, directory_file')
+
+        elif isinstance(directory_file, str):
+            format_file_out = cp_directory.get_extension(directory_file, point=False).lower()
+        else:
+            raise TypeError('directory_file')
+
+    elif isinstance(format_file, str):
+
+        if directory_file is None:
+
+            format_file_out = format_file.lower()
+        elif isinstance(directory_file, str):
+
+            format_file_out = format_file.lower()
+
+            format_file_tmp = cp_directory.get_extension(directory_file, point=False).lower()
+
+            if format_file_out != format_file_tmp:
+                raise ValueError('format_file or directory_file')
+        else:
+            raise TypeError('directory_file')
+
+    else:
+        raise TypeError('format_file')
+
+    return format_file_out
