@@ -1,6 +1,6 @@
 
 
-__all__ = ['image']
+__all__ = ['features', 'image', 'model_tools']
 
 
 def initiate(names_submodules=None):
