@@ -1,7 +1,8 @@
 from matplotlib import pyplot as plt
 from .save import *
+from .parameters import my_dpi
 
-__all__ = ['heatmaps', 'points', 'bars', 'save_figures', 'cmaps', 'colors']
+__all__ = ['heatmaps', 'points', 'bars', 'save_figures', 'cmaps', 'colors', 'my_dpi']
 
 
 def initiate(names_submodules=None):
