@@ -1,12 +1,17 @@
 from . import plt
 from .parameters import my_dpi
+from .. import directory as cp_directory
+import os
+
 
 __all__ = ['save_figures']
 
 
-def save_figures(id_figures=None, directories=None, formats=None):
+def save_figures(id_figures=None, directories=None, formats=None, close_figures=False):
+
     if id_figures is None:
         id_figures = plt.get_fignums()
+
     n_figures = len(id_figures)
 
     if directories is None:
@@ -28,9 +33,18 @@ def save_figures(id_figures=None, directories=None, formats=None):
     n_formats = len(formats)
     for i in range(n_figures):
 
-        plt.figure(id_figures[i])
+        figure_i = plt.figure(id_figures[i])
+
+        root_i = os.path.dirname(directories[i])
+
+        if len(root_i) > 0:
+            os.makedirs(root_i, exist_ok=True)
 
         for j in range(n_formats):
-            # plt.subplots_adjust(hspace=hspace)
-            directory_i_j = '.'.join([directories[i], formats[j]])
+
+            directory_i_j = cp_directory.add_extension(directories[i], formats[j])
             plt.savefig(directory_i_j, format=formats[j], dpi=my_dpi)
+
+        if close_figures:
+            plt.close(fig=figure_i)
+            figure_i = None
