@@ -1,9 +1,11 @@
+import math
+
 import numpy as np
 from scipy.stats import t
 # from scipy.stats.stats import _ttest_finish
-from ..combinations import n_conditions_to_combinations, trials_to_n_conditions
-from .. import array as cc_array
-from .. import maths as cc_maths
+from .. import combinations as cp_combinations
+from .. import array as cp_array
+from .. import maths as cp_maths
 
 
 def select_sum(ignore_nan=True):
@@ -15,12 +17,12 @@ def select_sum(ignore_nan=True):
 
 
 def counter_excluding_nan_keeping_dims(scores, axis_samples):
-    n_samples = np.sum(cc_maths.is_not_nan(scores), axis=axis_samples, keepdims=True)
+    n_samples = np.sum(cp_maths.is_not_nan(scores), axis=axis_samples, keepdims=True)
     return n_samples
 
 
 def counter_excluding_nan_not_keeping_dims(scores, axis_samples):
-    n_samples = np.sum(cc_maths.is_not_nan(scores), axis=axis_samples, keepdims=False)
+    n_samples = np.sum(cp_maths.is_not_nan(scores), axis=axis_samples, keepdims=False)
     return n_samples
 
 
@@ -54,7 +56,7 @@ def select_counter(ignore_nan=True, keepdims=False):
 
 
 def scores_to_diff_of_scores(scores, axis, delta=1, stride=1, keepdims=False):
-    # this function will go to cc.array in future versions
+    # this function will go to cp.array in future versions
 
     shape_scores = np.asarray(scores.shape)
     n_axes_scores = shape_scores.size
@@ -62,7 +64,7 @@ def scores_to_diff_of_scores(scores, axis, delta=1, stride=1, keepdims=False):
         axis += n_axes_scores
 
     n_conditions = shape_scores[axis]
-    index_0 = np.empty(n_axes_scores, dtype=object)
+    index_0 = np.empty(n_axes_scores, dtype='O')
     index_0[:] = slice(None)
     index_1 = np.copy(index_0)
 
@@ -121,7 +123,7 @@ def scores_to_n_samples(scores, axis_samples, keepdims=False, ignore_nan=True):
         n_axes_object_scores = shape_object_scores.size
         axes_object_scores = np.arange(n_axes_object_scores)
 
-        indexes_object = np.empty(n_axes_object_scores, dtype=object)
+        indexes_object = np.empty(n_axes_object_scores, dtype='O')
         indexes_object[:] = 0
         shape_scores = np.asarray(scores[tuple(indexes_object)].shape)
         n_axes_scores = shape_scores.size
@@ -135,13 +137,13 @@ def scores_to_n_samples(scores, axis_samples, keepdims=False, ignore_nan=True):
             shape_scores_tmp = shape_scores[np.arange(n_axes_scores) != axis_samples]
 
         shape_n_samples = np.append(shape_object_scores, shape_scores_tmp)
-        n_samples = np.empty(shape_n_samples, dtype=int)
+        n_samples = np.empty(shape_n_samples, dtype='i')
 
         n_axes_n_samples = shape_n_samples.size
-        indexes_n_samples = np.empty(n_axes_n_samples, dtype=object)
+        indexes_n_samples = np.empty(n_axes_n_samples, dtype='O')
         indexes_n_samples[:] = slice(None)
 
-        indexes_object = n_conditions_to_combinations(shape_object_scores)
+        indexes_object = cp_combinations.n_conditions_to_combinations(shape_object_scores)
         for indexes_object_i in indexes_object:
 
             indexes_n_samples[axes_object_scores] = indexes_object_i
@@ -151,7 +153,7 @@ def scores_to_n_samples(scores, axis_samples, keepdims=False, ignore_nan=True):
             n_samples[indexes_n_samples_tuple] = counter(scores[indexes_object_i_tuple], axis_samples=axis_samples)
 
             # n_samples[indexes_n_samples_tuple] = np.sum(
-            #     cc_maths.is_not_nan(scores[indexes_object_i_tuple]), axis=axis_samples, keepdims=keepdims)
+            #     cp_maths.is_not_nan(scores[indexes_object_i_tuple]), axis=axis_samples, keepdims=keepdims)
 
     else:
         shape_scores = np.asarray(scores.shape)
@@ -166,6 +168,14 @@ def scores_to_n_samples(scores, axis_samples, keepdims=False, ignore_nan=True):
 
 def samples_to_frequencies(data, axis_samples, axis_variables_table, exclude_values=False, values=-1):
 
+    shape_data = np.asarray(data.shape, dtype='i')
+    n_axes_data = shape_data.size
+
+    if axis_samples < 0:
+        axis_samples += n_axes_data
+    if axis_variables_table < 0:
+        axis_variables_table += n_axes_data
+
     if axis_samples < axis_variables_table:
         axis_comb = 0
     elif axis_samples > axis_variables_table:
@@ -174,38 +184,48 @@ def samples_to_frequencies(data, axis_samples, axis_variables_table, exclude_val
         raise ValueError('The following assumption is not met:\n'
                          '\taxis_samples \u003D axis_variables_table')
 
-    shape_data = np.asarray(data.shape, dtype=int)
-    n_axes_data = shape_data.size
     axes_data = np.arange(n_axes_data)
-    axes_others_and_samples = axes_data[axes_data != axis_variables_table]
-    axes_other_data = axes_others_and_samples[axes_others_and_samples != axis_samples]
+    axes_not_variables_data = axes_data[axes_data != axis_variables_table]
+    axes_other_data = axes_not_variables_data[axes_not_variables_data != axis_samples]
     n_axes_other_data = axes_other_data.size
     n_variables_table = shape_data[axis_variables_table]
 
-    indexes_data = np.empty(n_axes_data, dtype=object)
+    indexes_data = np.empty(n_axes_data, dtype='O')
     indexes_data[:] = slice(None)
-    indexes_data[axes_other_data] = 0
 
-    n_conditions = trials_to_n_conditions(
-        data[tuple(indexes_data)], axis_combinations=axis_comb, exclude_values=exclude_values, values=values)
+    # conditions = [None] * n_variables_table  # type: list
+    conditions = None  # type: # list
+
+    for indexes_axes_other in cp_combinations.n_conditions_to_combinations_on_the_fly(shape_data[axes_other_data]):
+
+        indexes_data[axes_other_data] = indexes_axes_other
+
+        conditions_tmp = cp_combinations.trials_to_conditions(
+            data[tuple(indexes_data)], axis_combinations=axis_comb, exclude_values=exclude_values, values=values)
+
+        if conditions is None:
+            conditions = conditions_tmp
+        else:
+            conditions = [np.unique(np.append(conditions[v], conditions_tmp[v])) for v in range(0, n_variables_table, 1)]
+
+    n_conditions = cp_combinations.conditions_to_n_conditions(conditions)
 
     n_axes_frequencies = n_axes_other_data + n_variables_table
-    shape_frequencies = np.empty(n_axes_frequencies, dtype=int)
+    shape_frequencies = np.empty(n_axes_frequencies, dtype='i')
     shape_frequencies[slice(n_axes_other_data)] = shape_data[axes_other_data]
     shape_frequencies[slice(n_axes_other_data, n_axes_frequencies)] = n_conditions
-    frequencies = np.empty(shape_frequencies, dtype=int)
-    frequencies[:] = 0
-    indexes_frequencies = np.empty(n_axes_frequencies, dtype=object)
+    frequencies = np.full(shape_frequencies, fill_value=0, dtype='i')
+    indexes_frequencies = np.empty(n_axes_frequencies, dtype='O')
 
-    combinations_others_and_samples = n_conditions_to_combinations(shape_data[axes_others_and_samples])
-    # combinations_others = combinations_others_and_samples[slice(None), axes_others_and_samples != axis_samples]
-    indexes_comb_c = axes_others_and_samples != axis_samples
-    for comb_c in combinations_others_and_samples:
+    combinations_not_variables = cp_combinations.n_conditions_to_combinations(shape_data[axes_not_variables_data])
+    # combinations_others = combinations_not_variables[slice(None), axes_not_variables_data != axis_samples]
+    indexes_comb_c = axes_not_variables_data != axis_samples
+    for comb_c in combinations_not_variables:
 
-        indexes_data[axes_others_and_samples] = comb_c
+        indexes_data[axes_not_variables_data] = comb_c
         data_c = data[tuple(indexes_data)]
         if exclude_values:
-            if all(cc_array.samples_in_arr1_are_not_in_arr2(data_c, values)):
+            if all(cp_array.samples_in_arr1_are_not_in_arr2(data_c, values)):
                 indexes_frequencies[slice(n_axes_other_data)] = comb_c[indexes_comb_c]
                 indexes_frequencies[slice(n_axes_other_data, n_axes_frequencies)] = data_c
                 frequencies[tuple(indexes_frequencies)] += 1
@@ -217,6 +237,108 @@ def samples_to_frequencies(data, axis_samples, axis_variables_table, exclude_val
     return frequencies
 
 
+def samples_to_local_frequencies(
+        data, axis_samples, axis_variables_table, size_local, stride=1, exclude_values=False, values=-1):
+
+    shape_data = np.asarray(data.shape, dtype='i')
+    n_axes_data = shape_data.size
+
+    if axis_samples < 0:
+        axis_samples += n_axes_data
+    if axis_variables_table < 0:
+        axis_variables_table += n_axes_data
+
+    if axis_samples < axis_variables_table:
+        axis_comb = 0
+    elif axis_samples > axis_variables_table:
+        axis_comb = 1
+    else:
+        raise ValueError('The following assumption is not met:\n'
+                         '\taxis_samples \u003D axis_variables_table')
+
+    axes_data = np.arange(n_axes_data, dtype='i')
+    axes_not_variables_data = axes_data[axes_data != axis_variables_table]
+    axes_other_data = axes_not_variables_data[axes_not_variables_data != axis_samples]
+    n_axes_other_data = axes_other_data.size
+    n_axes_not_variables_data = axes_not_variables_data.size
+    n_variables_table = shape_data[axis_variables_table]
+    I_in = data.shape[axis_samples]
+
+    indexes_data = np.asarray([slice(0, data.shape[a], 1) for a in range(0, n_axes_data, 1)], dtype='O')
+
+    # conditions = [None] * n_variables_table  # type: list
+    conditions = None  # type: # list
+
+    for indexes_axes_other in cp_combinations.n_conditions_to_combinations_on_the_fly(shape_data[axes_other_data]):
+
+        indexes_data[axes_other_data] = indexes_axes_other
+
+        conditions_tmp = cp_combinations.trials_to_conditions(
+            data[tuple(indexes_data)], axis_combinations=axis_comb, exclude_values=exclude_values, values=values)
+
+        if conditions is None:
+            conditions = conditions_tmp
+        else:
+            conditions = [np.unique(np.append(conditions[v], conditions_tmp[v])) for v in
+                          range(0, n_variables_table, 1)]
+
+    n_conditions = cp_combinations.conditions_to_n_conditions(conditions)
+
+    I_out = math.floor((I_in - size_local) / stride) + 1
+    n_axes_freq = n_axes_not_variables_data + n_variables_table
+
+    axes_freq = np.arange(n_axes_freq, dtype='i')
+
+    if axis_samples < axis_variables_table:
+        axis_samples_freq = axis_samples
+    elif axis_samples > axis_variables_table:
+        axis_samples_freq = axis_samples - 1
+    else:
+        raise ValueError('axis_samples == axis_variables_table')
+
+    n_axes_not_variables_freq = n_axes_not_variables_data
+    axes_not_variables_freq = np.arange(n_axes_not_variables_freq, dtype='i')
+    axes_other_freq = axes_not_variables_freq[axes_not_variables_freq != axis_samples_freq]
+
+    shape_frequencies = np.empty(n_axes_freq, dtype='i')
+    shape_frequencies[axes_other_freq] = shape_data[axes_other_data]
+    shape_frequencies[axis_samples_freq] = I_out
+    shape_frequencies[slice(n_axes_not_variables_freq, n_axes_freq)] = n_conditions
+    frequencies = np.full(shape_frequencies, fill_value=0, dtype='i')
+    indexes_frequencies = np.empty(n_axes_freq, dtype='O')
+
+    indexes_data = np.asarray([slice(0, data.shape[a], 1) for a in range(0, n_axes_data, 1)], dtype='O')
+    indexes_data_i = np.asarray(
+        [slice(0, data.shape[a], 1) if a != axis_samples else slice(0, size_local, 1)
+         for a in range(0, n_axes_data, 1)], dtype='O')
+
+    indexes_comb_c = axes_not_variables_data != axis_samples
+
+    for i, start_i in enumerate(range(0, I_in - (size_local - 1), stride)):
+
+        end_i = start_i + size_local
+        indexes_data[axis_samples] = slice(start_i, end_i, 1)
+
+        data_i = data[tuple(indexes_data)]
+
+        shape_data_i = np.asarray(data_i.shape, dtype='i')
+
+        indexes_frequencies[axis_samples_freq] = i
+
+        for comb_c in cp_combinations.n_conditions_to_combinations_on_the_fly(shape_data_i[axes_not_variables_data]):
+
+            indexes_data_i[axes_not_variables_data] = comb_c
+            data_ic = data_i[tuple(indexes_data_i)]
+            if exclude_values and any(cp_array.samples_in_arr1_are_in_arr2(data_ic, values)):
+                continue
+
+            indexes_frequencies[axes_other_freq] = comb_c[indexes_comb_c]
+            indexes_frequencies[slice(n_axes_not_variables_data, n_axes_freq)] = data_ic
+            frequencies[tuple(indexes_frequencies)] += 1
+
+    return frequencies
+
+
 def samples_to_probabilities(data, axis_samples, axis_variables_table):
 
     if axis_samples == axis_variables_table:
@@ -260,14 +382,14 @@ def scores_to_means(scores, axes, keepdims=False, ignore_nan=True):
         n_axes_object_scores = shape_object_scores.size
         axes_object_scores = np.arange(n_axes_object_scores)
 
-        indexes_object = np.empty(n_axes_object_scores, dtype=object)
+        indexes_object = np.empty(n_axes_object_scores, dtype='O')
         indexes_object[:] = 0
-        shape_scores = np.asarray(scores[tuple(indexes_object)].shape, dtype=int)
+        shape_scores = np.asarray(scores[tuple(indexes_object)].shape, dtype='i')
         n_axes_scores = shape_scores.size
         try:
             len(axes_means)
             # n_axes_means = len(axes_means)
-            axes_means = np.asarray(axes_means, dtype=int)
+            axes_means = np.asarray(axes_means, dtype='i')
             axes_means[axes_means < 0] += n_axes_scores
             # check point 1
             if np.sum(axes_means[0] == axes_means) > 1:
@@ -276,7 +398,7 @@ def scores_to_means(scores, axes, keepdims=False, ignore_nan=True):
         except TypeError:
             if axes_means < 0:
                 axes_means += n_axes_scores
-            axes_means = np.asarray([axes_means], dtype=int)
+            axes_means = np.asarray([axes_means], dtype='i')
             # n_axes_means = 1
 
         if keepdims:
@@ -284,17 +406,17 @@ def scores_to_means(scores, axes, keepdims=False, ignore_nan=True):
             shape_scores_tmp[axes_means] = 1
         else:
             axes_scores = np.arange(n_axes_scores)
-            shape_scores_tmp = shape_scores[np.logical_not(cc_array.samples_in_arr1_are_in_arr2(
+            shape_scores_tmp = shape_scores[np.logical_not(cp_array.samples_in_arr1_are_in_arr2(
                 axes_scores, axes_means))]
 
         shape_means = np.append(shape_object_scores, shape_scores_tmp)
-        means = np.empty(shape_means, dtype=float)
+        means = np.empty(shape_means, dtype='f')
 
         n_axes_means = shape_means.size
-        indexes_means = np.empty(n_axes_means, dtype=object)
+        indexes_means = np.empty(n_axes_means, dtype='O')
         indexes_means[:] = slice(None)
 
-        indexes_object = n_conditions_to_combinations(shape_object_scores)
+        indexes_object = cp_combinations.n_conditions_to_combinations(shape_object_scores)
         for indexes_object_i in indexes_object:
 
             indexes_means[axes_object_scores] = indexes_object_i
@@ -305,7 +427,7 @@ def scores_to_means(scores, axes, keepdims=False, ignore_nan=True):
             for a in axes_means:
                 n_samples = scores_to_n_samples(
                     scores_tmp_i, axis_samples=a, keepdims=keepdims, ignore_nan=ignore_nan)
-                # n_samples = np.sum(cc_maths.is_not_nan(scores_tmp_i), axis=a, keepdims=keepdims)
+                # n_samples = np.sum(cp_maths.is_not_nan(scores_tmp_i), axis=a, keepdims=keepdims)
 
                 sum_of_scores = sum(scores_tmp_i, axis=a, keepdims=keepdims)
                 scores_tmp_i = sum_of_scores / n_samples
@@ -314,13 +436,13 @@ def scores_to_means(scores, axes, keepdims=False, ignore_nan=True):
 
     else:
 
-        shape_scores = np.asarray(scores.shape, dtype=int)
+        shape_scores = np.asarray(scores.shape, dtype='i')
         n_axes_scores = shape_scores.size
 
         try:
             len(axes_means)
             # n_axes_means = len(axes_means)
-            axes_means = np.asarray(axes_means, dtype=int)
+            axes_means = np.asarray(axes_means, dtype='i')
             axes_means[axes_means < 0] += n_axes_scores
             # check point 1
             if np.sum(axes_means[0] == axes_means) > 1:
@@ -329,13 +451,13 @@ def scores_to_means(scores, axes, keepdims=False, ignore_nan=True):
         except TypeError:
             if axes_means < 0:
                 axes_means += n_axes_scores
-            axes_means = np.asarray([axes_means], dtype=int)
+            axes_means = np.asarray([axes_means], dtype='i')
             # n_axes_means = 1
         scores_tmp = scores
         for a in axes_means:
             n_samples = scores_to_n_samples(
                 scores_tmp, axis_samples=a, keepdims=keepdims, ignore_nan=ignore_nan)
-            # n_samples = np.sum(cc_maths.is_not_nan(scores_tmp), axis=a, keepdims=keepdims)
+            # n_samples = np.sum(cp_maths.is_not_nan(scores_tmp), axis=a, keepdims=keepdims)
             sum_of_scores = sum(scores_tmp, axis=a, keepdims=keepdims)
             scores_tmp = sum_of_scores / n_samples
         means = scores_tmp
@@ -353,7 +475,7 @@ def scores_to_variances(scores, axis_samples, keepdims=False, ignore_nan=True):
         n_axes_object_scores = shape_object_scores.size
         axes_object_scores = np.arange(n_axes_object_scores)
 
-        indexes_object = np.empty(n_axes_object_scores, dtype=object)
+        indexes_object = np.empty(n_axes_object_scores, dtype='O')
         indexes_object[:] = 0
         shape_scores = np.asarray(scores[tuple(indexes_object)].shape)
         n_axes_scores = shape_scores.size
@@ -367,13 +489,13 @@ def scores_to_variances(scores, axis_samples, keepdims=False, ignore_nan=True):
             shape_scores_tmp = shape_scores[np.arange(n_axes_scores) != axis_samples]
 
         shape_variances = np.append(shape_object_scores, shape_scores_tmp)
-        variances = np.empty(shape_variances, dtype=float)
+        variances = np.empty(shape_variances, dtype='f')
 
         n_axes_variances = shape_variances.size
-        indexes_variances = np.empty(n_axes_variances, dtype=object)
+        indexes_variances = np.empty(n_axes_variances, dtype='O')
         indexes_variances[:] = slice(None)
 
-        indexes_object = n_conditions_to_combinations(shape_object_scores)
+        indexes_object = cp_combinations.n_conditions_to_combinations(shape_object_scores)
         for indexes_object_i in indexes_object:
 
             indexes_variances[axes_object_scores] = indexes_object_i
@@ -382,7 +504,7 @@ def scores_to_variances(scores, axis_samples, keepdims=False, ignore_nan=True):
 
             n_samples = scores_to_n_samples(
                 scores[indexes_object_i_tuple], axis_samples=axis_samples, keepdims=True, ignore_nan=ignore_nan)
-            # n_samples = np.sum(cc_maths.is_not_nan(
+            # n_samples = np.sum(cp_maths.is_not_nan(
             #     scores[indexes_object_i_tuple]), axis=axis_samples, keepdims=True)
 
             sum_of_scores = sum(scores[indexes_object_i_tuple], axis=axis_samples, keepdims=True)
@@ -414,7 +536,7 @@ def scores_to_variances(scores, axis_samples, keepdims=False, ignore_nan=True):
             axis_samples += n_axes_scores
 
         n_samples = scores_to_n_samples(scores, axis_samples=axis_samples, keepdims=True, ignore_nan=ignore_nan)
-        # n_samples = np.sum(cc_maths.is_not_nan(scores), axis=axis_samples, keepdims=True)
+        # n_samples = np.sum(cp_maths.is_not_nan(scores), axis=axis_samples, keepdims=True)
         sum_of_scores = sum(scores, axis=axis_samples, keepdims=True)
         means = sum_of_scores / n_samples
         # means = np.nansum(scores, axis=axis_samples, keepdims=True) / n_samples
@@ -469,8 +591,8 @@ def scores_to_confidence_intervals(
     df = scores_to_df_of_variance_and_paired_t(scores, axis_samples, keepdims=keepdims, ignore_nan=ignore_nan)
 
     shape_df = np.asarray(df.shape)
-    indexes_df = n_conditions_to_combinations(shape_df)
-    t_critical = np.empty(shape_df, dtype=float)
+    indexes_df = cp_combinations.n_conditions_to_combinations(shape_df)
+    t_critical = np.empty(shape_df, dtype='f')
 
     for indexes_df_i in indexes_df:
 
@@ -489,16 +611,18 @@ def scores_to_confidence_intervals(
     return h
 
 
-def scores_to_local_mean(data, axis, n_neighbours, ignore_nan=True):
+def scores_to_local_mean(data, axis, size_local, stride=1, ignore_nan=True):
 
     n_axes = len(data.shape)
     if axis < 0:
         axis += n_axes
 
-    I = data.shape[axis]
+    I_in = data.shape[axis]
+    I_out = math.floor((I_in - size_local) / stride) + 1
 
     shape_means_local = list(data.shape)
-    shape_means_local[axis] = I - n_neighbours + 1
+
+    shape_means_local[axis] = I_out
 
     means_local = np.empty(shape_means_local, dtype='f')
 
@@ -510,9 +634,13 @@ def scores_to_local_mean(data, axis, n_neighbours, ignore_nan=True):
             indexes_means_local[a] = slice(0, means_local.shape[a], 1)
             indexes_data[a] = slice(0, data.shape[a], 1)
 
-    for i in range(0, I - n_neighbours + 1, 1):
+    for i, start_i in enumerate(range(0, I_in - (size_local - 1), stride)):
+
+        end_i = start_i + size_local
+        indexes_data[axis] = slice(start_i, end_i, 1)
+
         indexes_means_local[axis] = i
-        indexes_data[axis] = slice(i, i + n_neighbours, 1)
+
         means_local[tuple(indexes_means_local)] = scores_to_means(
             data[tuple(indexes_data)], axes=axis, keepdims=False, ignore_nan=ignore_nan)
 
@@ -521,8 +649,8 @@ def scores_to_local_mean(data, axis, n_neighbours, ignore_nan=True):
 
 def scores_to_value_frequencies(data, axis, value):
 
-    if cc_maths.is_nan(value):
-        arr_logical = cc_maths.is_nan(data)
+    if cp_maths.is_nan(value):
+        arr_logical = cp_maths.is_nan(data)
     else:
         arr_logical = data == value
 
