@@ -7,45 +7,49 @@ from .. import format as cc_format
 
 
 def exclude_trials_with_rt_out_of_range(
-        data, axes, indexes,
-        lower_limit, upper_limit, value=np.nan, copy=False):
-
-    try:
-        axes[0]
-        axes_ = axes
-    except TypeError:
-        axes_ = [axes]
-
-    try:
-        indexes[0]
-        indexes_ = indexes
-    except TypeError:
-        indexes_ = [indexes]
-
-    condition = np.empty(data.shape, dtype=bool)
-
-    n_axes = len(data.shape)
-    indexes_data = np.empty(n_axes, dtype='O')
-    indexes_condition = np.empty(n_axes, dtype='O')
-    j = 0
-    for a in range(n_axes):
-        if a in axes_:
-            try:
-                indexes_[j][0]
-            except TypeError:
-                if not isinstance(indexes_[j], slice):
-                    indexes_[j] = slice(indexes_[j], indexes_[j] + 1, 1)
-
-            indexes_data[a] = indexes_[j]
-            j += 1
-        else:
-            indexes_data[a] = slice(0, data.shape[a], 1)
-
-        indexes_condition[a] = slice(0, data.shape[a], 1)
-
-    data_to_check = data[tuple(indexes_data)]
-
-    condition[tuple(indexes_condition)] = np.logical_or(data_to_check < lower_limit, upper_limit < data_to_check)
+        data, lower_limit, upper_limit, axes=None, indexes=None, value=np.nan, copy=False):
+    
+    
+    if (axes is None) or (indexes is None):
+        condition = np.logical_or(data < lower_limit, upper_limit < data)
+    else:
+        
+        try:
+            len(axes)
+            axes_ = axes
+        except TypeError:
+            axes_ = [axes]
+    
+        try:
+            len(indexes)
+            indexes_ = indexes
+        except TypeError:
+            indexes_ = [indexes]
+    
+        condition = np.empty(data.shape, dtype=bool)
+    
+        n_axes = len(data.shape)
+        indexes_data = np.empty(n_axes, dtype='O')
+        indexes_condition = np.empty(n_axes, dtype='O')
+        j = 0
+        for a in range(n_axes):
+            if a in axes_:
+                try:
+                    len(indexes_[j])
+                except TypeError:
+                    if not isinstance(indexes_[j], slice):
+                        indexes_[j] = slice(indexes_[j], indexes_[j] + 1, 1)
+    
+                indexes_data[a] = indexes_[j]
+                j += 1
+            else:
+                indexes_data[a] = slice(0, data.shape[a], 1)
+    
+            indexes_condition[a] = slice(0, data.shape[a], 1)
+    
+        data_to_check = data[tuple(indexes_data)]
+    
+        condition[tuple(indexes_condition)] = np.logical_or(data_to_check < lower_limit, upper_limit < data_to_check)
 
     if copy:
         data_out = deepcopy(data)
@@ -300,6 +304,9 @@ def exclude_subs_with_higher_local_excluded_trials(
 
 def exclude_subs(data, axis, subs_exc):
 
+    if isinstance(data, (list, tuple)):
+        data = np.asarray(data)
+
     subs_all = np.arange(data.shape[axis])
     subs_keep = subs_all[cc_array.samples_in_arr1_are_not_in_arr2(subs_all, subs_exc, axis=0)]
 
