@@ -147,7 +147,7 @@ def csv_file_to_arrays(filename, rows, columns, dtype=None, encoding=None):
     arrays = [None] * n_arrays
     for a in range(n_arrays):
         indexes = tuple(cc_format.numeric_indexes_to_slice([rows[a], columns[a]]))
-        if (dtype[a] is None) or (dtype[a] == str) or ('U' in dtype[a][:3]):
+        if (dtype[a] is None) or (dtype[a] == str) or (dtype[a] == 'U'):
             arrays[a] = table[indexes]
         else:
             arrays[a] = table[indexes].astype(dtype[a])
@@ -217,10 +217,12 @@ def conditions_of_csv_files_to_arrays(conditions_of_directories, rows, columns,
         # array_csv_d = list(reader)
         array_csv_d = np.asarray(list(reader), dtype='U')
 
-    indexes_out = [None] * n_arrays
+    indexes_out = [None for a in range(n_arrays)]  # type: list
+    convert_to_U = [None for a in range(n_arrays)]  # type: list
     for a in range(n_arrays):
-        if dtype[a] is None:
-            dtype[a] = 'U'
+        if (dtype[a] is None) or (dtype[a] == 'U') or (dtype[a] == str):
+            dtype[a] = 'O'
+            convert_to_U[a] = True
 
         array_a_d = array_csv_d[indexes[a]]
         shape_array_a_d = np.asarray(array_a_d.shape, dtype=int)
@@ -229,8 +231,8 @@ def conditions_of_csv_files_to_arrays(conditions_of_directories, rows, columns,
 
         arrays[a] = np.empty(shape_array_a, dtype=dtype[a])
         n_axes_a = shape_array_a.size
-        indexes_out = np.empty(n_axes_a, dtype='O')
-        indexes_out[:] = slice(None)
+        indexes_out[a] = np.empty(n_axes_a, dtype='O')
+        indexes_out[a][:] = slice(None)
 
     for d in range(n_combinations_directories):
 
@@ -241,11 +243,16 @@ def conditions_of_csv_files_to_arrays(conditions_of_directories, rows, columns,
             reader = csv.reader(csvfile, delimiter=',')
             array_csv_d = np.asarray(list(reader), dtype='U')
 
-        indexes_out[slice(0, n_axes_directories_squeezed, 1)] = (
-            combinations_directories[d, axes_directories_squeezed])
-
         for a in range(n_arrays):
-            arrays[a][tuple(indexes_out)] = array_csv_d[indexes[a]]  # .astype(dtype[a])
+
+            indexes_out[a][slice(0, n_axes_directories_squeezed, 1)] = (
+                combinations_directories[d, axes_directories_squeezed])
+
+            arrays[a][tuple(indexes_out[a])] = array_csv_d[indexes[a]]  # .astype(dtype[a])
+
+    for a in range(n_arrays):
+        if convert_to_U[a]:
+            arrays[a] = arrays[a].astype('U')
 
     return arrays
 
